% -------------------------------------------------------------------------
% Replication of Figures 1 and 2 in
% Alloza, Gonzalo and Sanz (2024): Dynamic Effects of Persistent Shocks
% -------------------------------------------------------------------------

clear; clc; close all;

% Setting up initial parameters
H=21;
nobs    = 1*10^7;
y0      = 100;
rho     = 0.9;
nlagsY  = 1;
ymean   = 100;
B0      = 1.5;
B1      = 1;
trim    = 1;
z0      = 0;
zmean   = 0;
rhoZ    = 0.2;

y2_0    = 50;
rho2    = 0.7;
ymean2  = 50;
C0      = 1;
C1      = 0.5;

A2to1   = 0.3;  %Impact of y2_t-1 on y1_t
A1to2   = 0;    %Impact of y1_t-1 on y2_t

saveFIG = 1; %1=save figures, 0=don't

% Generate regressor X - no persistance
X=randn(nobs,1);
lagX = lagmatrix(X,1); 

% Generate regressor Z - with persistence
Z = zeros(nobs,1);
Z(1,1) = z0;
residZ = randn(nobs,1); % or use =X;

for i=2:nobs
    Z(i,1) = (1-rhoZ)*zmean + rhoZ*Z(i-1,1)+ residZ(i,1);
end
lagZ = lagmatrix(Z,1); 
fwdZ = lagmatrix(Z,-1);
fwdZ = fwdZ(trim+1:end,:);

% Generate outcome Y2 - second dep variable when shock is NOT persistent
Y2 = zeros(nobs,1);
Y2(1,1) = y2_0;
residual2 = randn(nobs,1);
for i=2:nobs
    Y2(i,1) = (1-rho2)*ymean2 + rho2*Y2(i-1,1) + C0*X(i,1)+ C1*lagX(i,1) + residual2(i,1);
end
lagY2 = lagmatrix(Y2,1); 

% Generate outcome Y - variable when shock is NOT persistent
Y = zeros(nobs,1);
Y(1,1) = y0;
residual = randn(nobs,1);
for i=2:nobs
    Y(i,1) = (1-rho)*ymean + rho*Y(i-1,1) + A2to1*Y2(i-1,1) + B0*X(i,1)+ B1*lagX(i,1) + residual(i,1);
end
lagY = lagmatrix(Y,1); 

% Generate outcome Q2 - second dep variable when shock IS persistent
Q2 = zeros(nobs,1);
Q2(1,1) = y2_0;
residua2l = randn(nobs,1);
for i=2:nobs
    Q2(i,1) = (1-rho2)*ymean2 + rho2*Q2(i-1,1) + C0*Z(i,1)+ C1*lagZ(i,1) + residual2(i,1);
end
lagQ2 = lagmatrix(Q2,1);

% Generate outcome Q - variable when shock IS persistent
Q = zeros(nobs,1);
Q(1,1) = y0;
residual = randn(nobs,1);
for i=2:nobs
    Q(i,1) = (1-rho)*ymean + rho*Q(i-1,1)+A2to1*Q2(i-1,1) + B0*Z(i,1)+ B1*lagZ(i,1) + residual(i,1);
end
lagQ = lagmatrix(Q,1);

%Preparing matrices for estimation
XX = [ones(nobs,1) X lagX lagY lagY2]; XX = XX(trim+1:end,:);
ZZ = [ones(nobs,1) Z lagZ lagQ lagQ2]; ZZ = ZZ(trim+1:end,:);

YY = Y(trim+1:end,:);
QQ = Q(trim+1:end,:);


%% True IRF
IR_true = zeros(H,1);
IR_true(1,:)  = B0;
IR_true(2,:)  = rho*IR_true(1,:) + B1;
for i = 3:H
    IR_true(i,:) = rho*IR_true(i-1,:);
end

%% LP-IRF with no persistance
for i=1:H
    yy = YY(i:end); 
    xx = XX(1:end-i+1,:);
    LPcoef = xx\yy;
    LPcoef_all(:,i)=LPcoef;
end
IR_LP = LPcoef_all(2,:);

%% LP-IRF with persistance
for i=1:H
    yy = QQ(i:end); 
    xx = ZZ(1:end-i+1,:);
    LPcoef_per = xx\yy;
    LP_per(:,i)=LPcoef_per;
end
IR_LP_per = LP_per(2,:);


%% VAR with the shock as an endogenous variable 
% case 1) VAR without persistence
Xreg = [ones(nobs,1) lagX lagY lagY2]; Xreg = Xreg(trim+1:end,:);
Yreg = [X(trim+1:end,:) YY Y2(trim+1:end,:)]; %note that the shock is ordered first

betaALL = Xreg\Yreg;
[T,n]  = size(Yreg);

Bmat0 = betaALL(1,:)'
Bmat1 = betaALL(2:end,:)'

u = Yreg - Xreg*betaALL;
SigU = u'*u/(n-1);
D = chol(SigU,'lower')

Dnorm = D./D(1,1)

delay=1; %number of lags
IRvar =zeros(H+delay,n);
IRvar(delay+1,:)  = Dnorm(:,1)';
for i = delay+2:delay+H
    IRvar(i,:) = IRvar(i-1,:)*Bmat1';
end
IRvarNOPER= IRvar(delay+1:end,:);

clear Xreg Yreg betaALL Bmat0 Bmat1 y SigU D Dnorm IRvar

% case 2) VAR with persistence
Xreg = [ones(nobs,1) lagZ lagQ lagQ2]; Xreg = Xreg(trim+1:end,:);
Yreg = [Z(trim+1:end,:) QQ Q2(trim+1:end,:)]; %note that the shock is ordered first

betaALL = Xreg\Yreg;
Bmat0 = betaALL(1,:)'
Bmat1 = betaALL(2:end,:)'

u = Yreg - Xreg*betaALL;
SigU = u'*u/(n-1);
D = chol(SigU,'lower')

Dnorm = D./D(1,1) %normalize because the shocks do not have one standard deviation


IRvar =zeros(H+delay,n);
IRvar(delay+1,:)  = Dnorm(:,1)';
for i = delay+2:delay+H
    IRvar(i,:) = IRvar(i-1,:)*Bmat1';
end
IRvarPER= IRvar(delay+1:end,:);

%% VAR with the shock as an exogenous variable 
%Case 1: with no persistence
nlagsX = 1;%H-1;
lagsEXO =  lagmatrix(X,1:nlagsX); 
Xreg = [ones(nobs,1) lagY lagY2 X lagsEXO ]; Xreg = Xreg(nlagsX+1:end,:);
Yreg = [Y(nlagsX+1:end,:) Y2(nlagsX+1:end,:)];

betaALL = Xreg\Yreg
[T,n]  = size(Yreg);

Bmat1 = betaALL(2:n+1,:)';
B_x0 = betaALL(n+2,:);
B_xlags = betaALL(end,:);

nlagsY=1;
IRvarX =zeros(H+nlagsY,n);
IRvarX(nlagsY+1,:)  = B_x0;
for ii = nlagsY+2:nlagsY+H
    IRvarX(ii,:) = IRvarX(ii-1,:)*Bmat1';
        if ii-nlagsY-1<nlagsX+1
            IRvarX(ii,:)=IRvarX(ii,:)+B_xlags(ii-nlagsY-1,:);
        end
end
IRvarX_NOPER= IRvarX(nlagsY+1:end,:);

clear Xreg Yreg betaALL B_mat1 B_x0 B_xlags IRvarX

%Case 2: with persistence
lagsEXO =  lagmatrix(Z,1:nlagsX); 
Xreg = [ones(nobs,1) lagQ lagQ2 Z lagsEXO ]; Xreg = Xreg(nlagsX+1:end,:);
Yreg = [Q(nlagsX+1:end,:) Q2(nlagsX+1:end,:)];

betaALL = Xreg\Yreg
[T,n]  = size(Yreg);

Bmat1 = betaALL(2:n+1,:)';
B_x0 = betaALL(n+2,:);
B_xlags = betaALL(end,:);

nlagsY=1;
IRvarX =zeros(H+nlagsY,n);
IRvarX(nlagsY+1,:)  = B_x0;
for ii = nlagsY+2:nlagsY+H
    IRvarX(ii,:) = IRvarX(ii-1,:)*Bmat1';
        if ii-nlagsY-1<nlagsX+1
            IRvarX(ii,:)=IRvarX(ii,:)+B_xlags(ii-nlagsY-1,:);
        end
end
IRvarX_PER= IRvarX(nlagsY+1:end,:);

%% FIGURES -- 
navyblue = [19,30,66]/255;
gris= [.85 0.85 0.85];
redix = [184,4,4]/255;
green = [0.466, 0.674, 0.188];
carrot = [0.9100    0.4100    0.1700]

%FIGURE VAR with shocks as ENDOGENOUS VARIABLE
figure
hold on
plot([0:H-1],IR_LP,'Color',navyblue,'Linewidth',3)
plot([0:H-1],IR_LP_per,'Color',redix,'Linewidth',3)
plot([0:H-1],IRvarNOPER(:,2),'--','Color',carrot,'Linewidth',3)
plot([0:H-1],IRvarPER(:,2),'--','Color',green,'Linewidth',3)
plot([0:H],zeros(1,H+1),'k')
axis([0 20  0 3.5])
legend('LP \gamma = 0','LP \gamma = 0.2','VAR \gamma = 0 ','VAR \gamma = 0.2 ')

if saveFIG == 1; print('-depsc2','Fig1a'); end

figure
hold on
plot([0:H-1],IRvarNOPER(:,1),'--','Color',carrot,'Linewidth',3)
plot([0:H-1],IRvarPER(:,1),'--','Color',green,'Linewidth',3)
plot([0:H],zeros(1,H+1),'k')
axis([0 20  0 1])
legend('VAR \gamma = 0 ','VAR \gamma = 0.2 ')

if saveFIG == 1; print('-depsc2','Fig1b'); end

figure
hold on
plot([0:H-1],IRvarNOPER(:,2),'Color',navyblue,'Linewidth',3)
plot([0:H-1],IRvarPER(:,2),'Color',redix,'Linewidth',3)
plot([0:H-1],IRvarX_NOPER(:,1),'--','Color',carrot,'Linewidth',3)
plot([0:H-1],IRvarX_PER(:,1),'--','Color',green,'Linewidth',2)
plot([0:H],zeros(1,H+1),'k')
axis([0 20  0 3.5])
legend('VAR \gamma = 0 ','VAR \gamma = 0.2 ','VAR-X \gamma = 0 ','VAR-X \gamma = 0.2 ')

if saveFIG == 1; print('-depsc2','Fig2'); end




