% -------------------------------------------------------------------------
% Replication of Figures B1, B2 and B4 in
% Alloza, Gonzalo and Sanz (2024): Dynamic Effects of Persistent Shocks
% -------------------------------------------------------------------------
clear; clc; close all;

H=21;
nobs    = 10^7; 
y0      = 100;
rho     = 0.9;
nlagsY  = 1;
ymean   = 100;
B0      = 1.5;
B1      = 1;
trim    = 1;
z0      = 0;
zmean   = 0;
rhoZ    = 0.2;

saveFIG = 1; %1=save figures, 0=don't

% Generate regressor X - no persistance
X=randn(nobs,1);
lagX = lagmatrix(X,1); 

% Generate regressor Z - with persistence
Z = zeros(nobs,1);
Z(1,1) = z0;
residZ = randn(nobs,1); % or use =X;

for i=2:nobs
    Z(i,1) = (1-rhoZ)*zmean + rhoZ*Z(i-1,1)+ residZ(i,1);
end
lagZ = lagmatrix(Z,1); 
fwdZ = lagmatrix(Z,-1);
fwdZ = fwdZ(trim+1:end,:);

    % Obtain residual from Z_t = beta Z_t-1 + residual
    Z_x = lagmatrix(Z,1); Z_x = Z_x(2:end);
    Z_y = Z(2:end);
    Z_resid = Z_y-(Z_y\Z_x)*Z_x;

% Generate outcome Y - variable when shock is NOT persistent
Y = zeros(nobs,1);
Y(1,1) = y0;
residual = randn(nobs,1);

for i=2:nobs
    Y(i,1) = (1-rho)*ymean + rho*Y(i-1,1) + B0*X(i,1)+ B1*lagX(i,1) + residual(i,1);
end
lagY = lagmatrix(Y,1); 

% Generate outcome Q - variable when shock IS persistent
Q = zeros(nobs,1);
Q(1,1) = y0;
residual = randn(nobs,1);

for i=2:nobs
    Q(i,1) = (1-rho)*ymean + rho*Q(i-1,1) + B0*Z(i,1)+ B1*lagZ(i,1) + residual(i,1);
end
lagQ = lagmatrix(Q,1);

%Preparing matrices for estimation
XX = [ones(nobs,1) X lagX lagY]; XX = XX(trim+1:end,:);
ZZ = [ones(nobs,1) Z lagZ lagQ]; ZZ = ZZ(trim+1:end,:);
YY = Y(trim+1:end,:);
QQ = Q(trim+1:end,:);

%% True IRF
IR_true = zeros(H,1);
IR_true(1,:)  = B0;
IR_true(2,:)  = rho*IR_true(1,:) + B1;
for i = 3:H
    IR_true(i,:) = rho*IR_true(i-1,:);
end

%% LP-IRF with no persistance
for i=1:H
    yy = YY(i:end); 
    xx = XX(1:end-i+1,:);
    LPcoef = xx\yy;
    LPcoef_all(:,i)=LPcoef;
end
IR_LP = LPcoef_all(2,:);

%% LP-IRF with persistance
for i=1:H
    yy = QQ(i:end); 
    xx = ZZ(1:end-i+1,:);
    LPcoef_per = xx\yy;
    LP_per(:,i)=LPcoef_per;
end
IR_LP_per = LP_per(2,:);

%% LP-IRF with persistance but including leads
ZZ_fwd = [ZZ fwdZ];
ZZ_fwd = ZZ_fwd(1:end-1,:); QQ_fwd=QQ(1:end-1,:); 
for i=1:H
    yy = QQ_fwd(i:end); 
    xx = ZZ_fwd(1:end-i+1,:);
    LPcoef_per_cor = xx\yy;
    LP_percor(:,i)=LPcoef_per_cor;
end
IR_LP_percor = LP_percor(2,:);

%% LP-IRF with normal persistance and GENERATED REGRESSOR
ZZ_resid = [ones(nobs-1,1) Z_resid]; ZZ_resid = ZZ_resid(trim+1:end,:);
QQ_resid = Q(3:end);
for i=1:H
    yy = QQ_resid(i:end); 
    xx = ZZ_resid(1:end-i+1,:);
    LPcoef_per_res = xx\yy;
    LP_per_res(:,i)=LPcoef_per_res;
end
IR_LP_per_res = LP_per_res(2,:);

%% MA-IRF with no persistance
lagX_h = lagmatrix(X,1:H-1); 
X_h = [ones(nobs,1) X lagX_h]; 
X_h = X_h(H-1+1:end,:);
dY = Y; 
dY = dY(H-1+1:end,:);
betas_noper= X_h\dY;
IRF_MA_noper = betas_noper(2:end); 

%% MA-IRF with persistance
lagZ_h = lagmatrix(Z,1:H-1+1);  
Z_h = [ones(nobs,1) Z lagZ_h(:,1:H-1)]; 
Z_h = Z_h(H-1+1:end,:);
dQ = Q;
dQ = dQ(H-1+1:end,:);
betas_withper= Z_h\dQ;
IRF_MA_withper = (betas_withper(2:end));

%% MA-IRF with persistance BUT using residual as generated regressor
lagZr_h = lagmatrix(Z_resid,1:H-1); 
Zr_h = [ones(nobs-1,1) Z_resid lagZr_h]; 
Zr_h = Zr_h(H-1+1:end,:);
dQ_res = Q;
dQ_res = dQ_res(H-1+1+1:end,:);
betas_withper_res= Zr_h\dQ_res;
IRF_MA_withper_res = betas_withper_res(2:end);

%% FIGURES -- 
navyblue = [19,30,66]/255;
gris= [.85 0.85 0.85];
redix = [184,4,4]/255;
green = [0.466, 0.674, 0.188];

figure
hold on
plot([0:H-1],IR_LP,'Color',navyblue,'Linewidth',3)
plot([0:H-1],IR_LP_per,'Color',redix,'Linewidth',3)
plot([0:H-1],IR_LP_percor,'--','Color',gris,'Linewidth',3)
plot([0:H],zeros(1,H+1),'k')
axis([0 20  0 3])
legend('\gamma = 0','\gamma \neq 0','\gamma \neq 0 & leads')
if saveFIG == 1; print('-depsc2','Figure B1'); end

figure
hold on
plot([0:H-1],IRF_MA_noper,'Color',navyblue,'Linewidth',3)
plot([0:H-1],IRF_MA_withper,'--','Color',gris,'Linewidth',3)
plot([0:H-1],IRF_MA_withper_res,'Color',redix,'Linewidth',3)
plot([0:H],zeros(1,H+1),'k')
axis([0 20  0 3])
leg1 = legend('$\gamma = 0$','$\gamma \neq 0$','$\gamma \neq 0 \ \& \ \hat{\varepsilon_t}$');
set(leg1,'Interpreter','latex');
if saveFIG == 1; print('-depsc2','Figure B2'); end

figure
hold on
plot([0:H-1],IR_true,'Color',navyblue,'Linewidth',3)
plot([0:H-1],IR_LP_per,'Color',redix,'Linewidth',3)
plot([0:H-1],IR_LP_per_res,'--','Color',gris,'Linewidth',3)
plot([0:H],zeros(1,H+1),'k')
axis([0 20  0 3])
leg1 = legend('$\gamma = 0$','$\gamma \neq 0$','$\gamma \neq 0 \ \& \ \hat{\varepsilon_t}$');
set(leg1,'Interpreter','latex');
if saveFIG == 1; print('-depsc2','Figure B4'); end


